"use strict";
// tslint:disable:no-console max-classes-per-file
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ansi = require("ansi-escapes");
const clime_1 = require("clime");
const fs = require("fs-extra");
const path = require("path");
const WebpackError_1 = require("../Errors/WebpackError");
const WebpackBuildError_1 = require("../Errors/WebpackBuildError");
const SPAGenerator_1 = require("../Modes/SPAGenerator");
const HTMLGenerator_1 = require("../Modes/HTMLGenerator");
const StringTools_1 = require("../../Common/Tools/StringTools");
const Util_1 = require("../../Common/Config/Util");
class CLICommandOptions extends clime_1.Options {
}
tslib_1.__decorate([
    clime_1.option({ flag: 'd', description: 'configuration directory', validator: (value) => fs.pathExistsSync(path.resolve(process.cwd(), value)) }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "configDir", void 0);
tslib_1.__decorate([
    clime_1.option({ flag: 'o', description: 'output directory' }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "outDir", void 0);
tslib_1.__decorate([
    clime_1.option({ flag: 'm', description: 'output mode', default: 'html' }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "mode", void 0);
tslib_1.__decorate([
    clime_1.option({ flag: 'r', description: 'render mode', default: 'htmlSuffix', validator: value => ['htmlSuffix', 'subDirectories', 'htaccess'].includes(value) }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "routerMode", void 0);
tslib_1.__decorate([
    clime_1.option({ flag: 'b', description: 'base URL', default: '' }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "baseUrl", void 0);
tslib_1.__decorate([
    clime_1.option({ description: 'Git branch', default: 'master' }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "repoBranch", void 0);
tslib_1.__decorate([
    clime_1.option({ description: 'index file' }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "indexFile", void 0);
tslib_1.__decorate([
    clime_1.option({ description: 'index title' }),
    tslib_1.__metadata("design:type", String)
], CLICommandOptions.prototype, "indexTitle", void 0);
exports.CLICommandOptions = CLICommandOptions;
let CLICommand = class CLICommand extends clime_1.Command {
    execute(inputFolders, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            process.env.NODE_ENV = 'production';
            const cwd = process.cwd();
            let configDir = null;
            if (options.configDir) {
                // check already made in validatior
                configDir = path.resolve(cwd, options.configDir);
            }
            else {
                const defaultConfigDir = path.resolve(cwd, 'docs');
                if (yield fs.pathExists(defaultConfigDir)) {
                    configDir = defaultConfigDir;
                }
            }
            let importedConfig = null;
            let indexFile = null;
            if (configDir) {
                const configFile = path.join(configDir, 'config.json');
                if (!(yield fs.pathExists(configFile))) {
                    throw new clime_1.ExpectedError('configuration directory was found but there is no config.json file in it');
                }
                try {
                    importedConfig = (yield fs.readJSON(configFile));
                }
                catch (e) {
                    throw new clime_1.ExpectedError(`error reading ${configFile} as JSON: ${e.message}`);
                }
                indexFile = options.indexFile || Util_1.getConfigValue(importedConfig, 'indexFile');
                if (indexFile) {
                    indexFile = path.join(configDir, indexFile);
                    if (!(yield fs.pathExists(indexFile))) {
                        throw new clime_1.ExpectedError('given index file does not exist in configuration directory');
                    }
                }
            }
            if (!indexFile) {
                indexFile = path.join(cwd, 'README.md');
                if (!(yield fs.pathExists(indexFile))) {
                    throw new clime_1.ExpectedError('there was neither a given index file nor a README.md in the root of the project');
                }
            }
            let generator;
            const generatorConfig = {
                configDir,
                mode: options.mode || Util_1.getConfigValue(importedConfig, 'mode') || 'html',
                routerMode: options.routerMode || Util_1.getConfigValue(importedConfig, 'routerMode') || 'htmlSuffix',
                inputDirs: inputFolders.length ? inputFolders : Util_1.getConfigValue(importedConfig, 'inputDirs', true),
                outputDir: options.outDir || Util_1.getConfigValue(importedConfig, 'outputDir', true),
                baseUrl: StringTools_1.removeSlash(options.baseUrl || Util_1.getConfigValue(importedConfig, 'baseUrl', true)),
                baseDir: cwd,
                repoUser: Util_1.getConfigValue(importedConfig, 'repoUser'),
                repoName: Util_1.getConfigValue(importedConfig, 'repoName'),
                repoBranch: options.repoBranch,
                indexTitle: options.indexTitle || Util_1.getConfigValue(importedConfig, 'indexTitle') || 'Welcome',
                indexFile: indexFile,
                categories: Util_1.getConfigValue(importedConfig, 'categories') || undefined,
                webpackProgressCallback: (percentage, msg, moduleProgress) => {
                    process.stdout.write(`${ansi.eraseLine}\rcompiling with webpack... ${percentage * 100}%`);
                    if (moduleProgress) {
                        process.stdout.write(` (${moduleProgress})`);
                    }
                }
            };
            switch (options.mode) {
                case 'spa': {
                    generator = new SPAGenerator_1.default(generatorConfig);
                    break;
                }
                case 'html': {
                    generator = new HTMLGenerator_1.default(generatorConfig);
                    break;
                }
                default: {
                    throw new Error(`Generator '${options.mode}' not found`);
                }
            }
            const data = generator.createReferenceStructure();
            try {
                yield generator.generate(data);
                process.exit(0);
            }
            catch (e) {
                if (e instanceof WebpackError_1.default) {
                    process.stderr.write('\nerror building with webpack:\n');
                    console.error(e.originalError);
                    process.exit(1);
                }
                else if (e instanceof WebpackBuildError_1.default) {
                    process.stderr.write('\nerror building with webpack:\n');
                    console.error(e.stats.toString());
                    process.exit(1);
                }
                else {
                    process.stderr.write('\nerror building the documentation:\n');
                    console.error(e);
                    process.exit(1);
                }
            }
        });
    }
};
tslib_1.__decorate([
    tslib_1.__param(0, clime_1.params({ type: String, description: 'input directories' })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Array, CLICommandOptions]),
    tslib_1.__metadata("design:returntype", Promise)
], CLICommand.prototype, "execute", null);
CLICommand = tslib_1.__decorate([
    clime_1.command()
], CLICommand);
exports.default = CLICommand;
//# sourceMappingURL=default.js.map